/********************************************************************** * 
 * @project Fusion
 * @revision $Id: Measure.js,v 1.37 2007/07/04 21:25:54 assefa Exp $
 * @purpose Measure widget
 * @author pspencer@dmsolutions.ca
 * Copyright (c) 2007 DM Solutions Group Inc.
 *****************************************************************************
 * This code shall not be copied or used without the expressed written consent
 * of DM Solutions Group Inc.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * ********************************************************************
 *
 * The Measure widget allows the user to measure distances or areas on the map in
 * one or more segments. Area is positive if measured clockwise.
 *
 * Minimal XML Tag:
 *
 * <Command xsi:type="FusionCommandType">
 *    <Name>Measure</Name>
 *    <Label></Label>
 *    <Tooltip>Measure distance and/or areas</Tooltip>
 *    <!-- Kind of measure: distance, area, both -->
 *    <MeasureType>both</MeasureType>
 *    <ImageURL>images/icons/measure.png</ImageURL>
 *    <TargetViewer>All</TargetViewer>
 *    <Action>Measure</Action>
 * </Command>
 *
 * Additionnal Tags:
 * <DistanceRoundValue>       :Fixed value for distance display, 2 => 0.00, 3 => 0.000,
 *                             but -2 => 1221 -> 1200
 * <AreaRoundValue>           :Fixed value for area display, 2 => 0.00, 3 => 0.000,
 *                             but -2 => 1221 -> 1200
 * <MeasureTooltipContainer>  :The ID of an existing <div>, default: MeasureContainerBox,
 *                             which is styled in css file (see bellow)
 * <MeasureTooltipType>       :Default value: dynamic, could static too.
 *   <MeasureTipPositionTop>  :When static above given, then give here the Top position
 *   <MeasureTipPositionLeft> :When static above given, then give here the Left position
 * 
 * You may want to customize the canvas distance and area appearance, please use these tags:
 *
 * <AreaFillStyle>            :Fill color rgba for area drawing: sample value: rgba(0,255,0,0.3)
 * <AreaLineStyleWidth>       :Border width in pixel for area drawing: sample value: 2
 * <AreaLineStyleColor>       :Border color rgba for area drawing.
 * <DistanceNormalStyleWidth> :Line width in pixel for distance drawing.
 * <DistanceNormalStyleColor> :Line color rgba for distance drawing.
 * <DistanceEditStyleWidth>   :Line width in pixel for current distance drawing.
 * <DistanceEditStyleColor>   :Line color rgba for current distance drawing.
 * <AreaEditStyleWidth>       :Line width in pixel for current area drawing.
 * <AreaEditStyleColor>       :Line color rgba for current area drawing.
 *
 * Place this in your css file, style of the Tooltip (Here fusion style)
 *
 * #MeasureContainerBox {
 *    background-image: url('./images/headerbg.gif');
 *    background-repeat: repeat-x;
 *    font: 11px tahoma;
 *    color: #FFFFFF;
 *    position: absolute;
 *    width: 140px;
 *    float: left;
 *    border: solid #000000 2px;
 *    display : 'none';
 *    visibility : hidden;
 *    filter:alpha(opacity=75);
 *    -moz-opacity:0.75;
 *    opacity: 0.75;
 * }   
 *
 * Distances for the current segment and the total distance are advertised
 * through the Fusion.Event.RULER_DISTANCE_CHANGED event. Only tested with meter unit
 * for now. You may want to change the htlm text to format the 
 * tooltip to match your desire: See Vars: dTipStr and aTipStr below.
 * 
 * **********************************************************************/

//Enum declaration
var EmType = { distance:0, area:1, both:2 };

Fusion.Event.RULER_DISTANCE_CHANGED = Fusion.Event.lastEventId++;
Fusion.Widget.Measure = Class.create();
Fusion.Widget.Measure.prototype = {
    isDigitizing: false,
    //distance of each segment
    aDistances: [],
    //Array of points used to compute area
    aAreaFirstPoint: null,
    //cumulativeDistance
    cumulativeDistance: 0,
    lastDistance: 0,
    //for areas
    //cumulativeArea
    cumulativeArea: 0,
    lastArea: 0,
    aAreas: [],
    
    /* the units to display distances in */
    units: Fusion.UNKNOWN,

    /* Type of measure: values = disance, area or both, default: both*/
    mType: null,

    /* Precision of the distance displayed */
    distPrecision: 0,
    
    /* Precision of the area displayed */
    areaPrecision: 0,
    
    /* an HTML container to put the current distance in */
    measureTip: null,
    
    /*debugging purpose*/
    message:"no message",
    
    /* Static position of Tooltip Box TOP */
    measureTipPositionTop: null,
   
    /* Static position of Tooltip Box LEFT */ 
    measureTipPositionLeft: null,
    
    /* Tooltip appearance: static or dynamic */
    tooltipType: '',

    /* Style for the distance line used for distance draw */   
    distanceNormalStyle: null,

    /* Style for the polygon used for area draw */   
    fillStyle: null,

    /* Style for the polygon line used for area draw */    
    polygonLineStyle: null,
    
    /* Style of the line "Stuck" at the mouse when polygon editing */    
    polygonEditStyle: null,

    /* Style of the line "Stuck" at the mouse when line editing */    
    lineEditStyle: null,
  
    initialize : function(oCommand) {
         
        /*console.log('Ruler.initialize');*/
        Object.inheritFrom(this, Fusion.Widget.prototype, ['Measure', true, oCommand]);
        Object.inheritFrom(this, Fusion.Tool.ButtonBase.prototype, []);
        Object.inheritFrom(this, Fusion.Tool.Canvas.prototype, []);
        this.setMap(oCommand.getMap());
        this.asCursor = ['crosshair'];
        var json = oCommand.jsonNode;
        this.units = (json.Units && (json.Units[0] != '')) ?  Fusion.unitFromName(json.Units[0]): this.units;
        this.distPrecision = json.DistanceRoundValue ? json.DistanceRoundValue[0] : 2;
        this.areaPrecision = json.AreaRoundValue ? json.AreaRoundValue[0] : 2;        
        //init measure type
        if(json.MeasureType){
            if(json.MeasureType[0] == 'distance'){
            this.mType = EmType.distance;
            }
            if(json.MeasureType[0] == 'area'){
            this.mType = EmType.area;
            }
            if(json.MeasureType[0] == 'both'){
            this.mType = EmType.both;
            }
        }else{
            this.mType = EmType.both;
        }
        var container = json.MeasureTooltipContainer ? json.MeasureTooltipContainer[0] : 'MeasureContainerBox';
        //Here are the canvas style definition
        var areaFillStyle = json.AreaFillStyle ? json.AreaFillStyle[0] : 'rgba(0,0,255, 0.3)';
        var areaLineStyleWidth = json.AreaLineStyleWidth ? json.AreaLineStyleWidth[0] : 2;
        var areaLineStyleColor = json.AreaLineStyleColor ? json.AreaLineStyleColor[0] : 'rgba(0,0,255,0.3)';     
        var distanceNormalStyleWidth = json.DistanceNormalStyleWidth ? json.DistanceNormalStyleWidth[0] : 2;
        var distanceNormalStyleColor = json.DistanceNormalStyleColor ? json.DistanceNormalStyleColor[0] : 'rgba(255,0,0,1.0)';    
        var distanceEditStyleWidth = json.DistanceEditStyleWidth ? json.DistanceEditStyleWidth[0] : 2;
        var distanceEditStyleColor = json.DistanceEditStyleColor ? json.DistanceEditStyleColor[0] : 'rgba(255,0,0,0.3)';
        var areaEditStyleWidth = json.AreaEditStyleWidth ? json.AreaEditStyleWidth[0] : 2;
        var areaEditStyleColor = json.AreaEditStyleColor ? json.AreaEditStyleColor[0] : 'rgba(0,0,255,0.3)';      
        this.fillStyle = new Fusion.Tool.Canvas.Style({fillStyle:areaFillStyle});
        this.polygonLineStyle = new Fusion.Tool.Canvas.Style({lineWidth:areaLineStyleWidth,strokeStyle:areaLineStyleColor});  	
        this.distanceNormalStyle = new Fusion.Tool.Canvas.Style({lineWidth:distanceNormalStyleWidth, strokeStyle:distanceNormalStyleColor});      
        this.distanceEditStyle = new Fusion.Tool.Canvas.Style({lineWidth:distanceEditStyleWidth, strokeStyle:distanceEditStyleColor}); 
        this.polygonEditStyle = new Fusion.Tool.Canvas.Style({lineWidth:areaEditStyleWidth, strokeStyle:areaEditStyleColor});        
        //set default container if not specified 
        var cont = document.getElementById(container);
        if (cont == null){
        cont = document.createElement('div');
        cont.id = container;
        }  
        if (cont != '') {
            this.measureTip = $(cont);
        }
        if (this.measureTip) {
            this.tooltipType = json.MeasureTooltipType ? json.MeasureTooltipType[0] : 'dynamic';
            if (this.tooltipType == 'dynamic') {
                var oDomElem =  this.getMap().getDomObj();
                oDomElem.appendChild(this.measureTip);
                this.measureTip.style.position = 'absolute';
                this.measureTip.style.display = 'none';
                this.measureTip.style.top = '0px';
                this.measureTip.style.left = '0px';
                this.measureTip.style.zIndex = 101;
            }
            if (this.tooltipType == 'static') {
            	this.measureTipPositionTop = json.MeasureTipPositionTop ? json.MeasureTipPositionTop[0] : 100;
                this.measureTipPositionLeft = json.MeasureTipPositionLeft ? json.MeasureTipPositionLeft[0] : 15;                                
                var oDomElem =  this.getMap().getDomObj();
                oDomElem.appendChild(this.measureTip);
                this.measureTip.style.position = 'absolute';
                this.measureTip.style.display = 'none';
                this.measureTip.style.top = this.measureTipPositionTop;
                this.measureTip.style.left = this.measureTipPositionLeft;
                this.measureTip.style.zIndex = 101;
            }
        }
        this.registerEventID(Fusion.Event.RULER_DISTANCE_CHANGED);
        this.getMap().registerForEvent(Fusion.Event.MAP_EXTENTS_CHANGED, this.resetCanvas.bind(this));
        this.keyHandler = this.onKeyPress.bind(this);
    },
    
    onKeyPress: function(e) {
        //console.log('Rule::onKeyPress');
        var charCode = (e.charCode ) ? e.charCode : ((e.keyCode) ? e.keyCode : e.which);
        //console.log(charCode);
        if (charCode == Event.KEY_ESC) {
            this.resetCanvas();
        } 
    },
    
    /**
     * (public) activate()
     *
     * activate the measure tool
     */
    activateTool: function() {
        this.getMap().activateWidget(this);
        this._oButton.activateTool();
    },

    /**
     * (public) initVars()
     *
     * reset area and/or distance vars
     */    
    initVars: function() {
    	if(this.mType == EmType.distance || this.mType == EmType.both)
    	{
            this.aDistances = [];
            this.cumulativeDistance = 0;
            this.lastDistance = 0;
        }
        if(this.mType == EmType.area || this.mType == EmType.both)
        {
            this.aAreas = [];
            this.cumulativeArea = 0;
            this.lastArea = 0;
            this.aAreaFirstPoint = null;
        }
    },
    
    activate: function() {
        this.activateCanvas();
        this.initVars();
        this.triggerEvent(Fusion.Event.RULER_DISTANCE_CHANGED, this);
        Event.observe(document,"keypress",this.keyHandler);
    },
    
    /**
     * (public) deactivate()
     *
     * deactivate the ruler tool
     */
    deactivate: function() {
        //console.log('Ruler.deactivate');
        Event.stopObserving(document, 'keypress', this.keyHandler);           
        this._oButton.deactivateTool();
        this.deactivateCanvas();
        this.resetCanvas();
    },
    
    resetCanvas: function() {
        if (this.isDigitizing) {
            this.isDigitizing = false;
        }
        this.clearContext();
        this.initVars();
        this.triggerEvent(Fusion.Event.RULER_DISTANCE_CHANGED, this);
        if (this.measureTip) {
            this.updateTip(null);
        }
    },
    
    /**
     * (public) drawFeature(seg, feature, section, e)
     *
     * draw a segment in a specified feature, depending 
     * on which event has occured
     *
     * @param seg Current segment object to draw
     * @param feature Canvas.Polygon or Canvas.Line object 
     * @param section Current state, see in code
     * @param e Event The event that happened on the mapObj
     */    
    drawFeature: function(seg, feature, section, e) {
    
        //1)    mouseDown + !this.isDigitizing 
        if(section == 1){     
            seg.setEditing(true);              
            feature.addSegment(seg);                 
        }
        //2)    mouseDown + this.isDigitizing
        if(section == 2){ 
            seg.setEditing(false);
            seg = feature.extendLine();
            seg.setEditing(true);
        }        
        //3)    mouse move    
        if(section == 3){         
            var p = this.getMap().getEventPosition(e);
            var gp = this.getMap().pixToGeo(p.x, p.y);            
            seg.to.set(gp.x,gp.y);
        }
        //4)    dbl click
        if(section == 4){   
            var p = this.getMap().getEventPosition(e);
            var gp = this.getMap().pixToGeo(p.x, p.y);   
            seg.setEditing(false);
            seg.to.set(gp.x,gp.y);
        }
        if(feature == this.distPolyline && this.mType == EmType.both){
            feature.draw(this.context);
        }else{
            this.clearContext(); 
            feature.draw(this.context);         
        }
    },    
      
    /**
     * (public) mouseDown(e)
     *
     * handle the mouse down event
     *
     * @param e Event the event that happened on the mapObj
     */
    mouseDown: function(e) {  	
        if (Event.isLeftClick(e)) {
            var map = this.getMap();
            var p = map.getEventPosition(e);
            var gp = map.pixToGeo(p.x, p.y);
            //if not digitizing
            if (!this.isDigitizing) {
                var from = new Fusion.Tool.Canvas.Node(gp.x,gp.y, map);
                var to = new Fusion.Tool.Canvas.Node(gp.x,gp.y, map);
                var seg = new Fusion.Tool.Canvas.Segment(from,to);                             
                //Draw a polygon
                if(this.mType == EmType.area){
                    this.Polygon = new Fusion.Tool.Canvas.Polygon(map);
                    this.Polygon.fillStyle = this.fillStyle;
                    this.Polygon.lineStyle = this.polygonLineStyle;        
                    this.drawFeature(seg, this.Polygon, 1);   
                }              
                //Draw distance border line
                if(this.mType == EmType.distance){
                    this.distPolyline = new Fusion.Tool.Canvas.Line(map);
                    this.drawFeature(seg, this.distPolyline, 1);
                }
                //Draw both
                if(this.mType == EmType.both){
                    this.Polygon = new Fusion.Tool.Canvas.Polygon(map);
                    this.Polygon.fillStyle = this.fillStyle;
                    this.Polygon.lineStyle = this.polygonLineStyle;        
                    this.distPolyline = new Fusion.Tool.Canvas.Line(map);
                    this.drawFeature(seg, this.Polygon, 1);
                    this.drawFeature(seg, this.distPolyline, 1);
                }                                     
                this.initVars();               
                initialPoint = new Fusion.Tool.Canvas.Node(gp.x,gp.y, map);
                if(this.mType == EmType.area || this.mType == EmType.both)
                    this.aAreaFirstPoint = initialPoint;
                this.isDigitizing = true;                                  
            //if digitizing
            } else {                 
                if(this.mType == EmType.area || this.mType == EmType.both){
                    var seg2 = this.Polygon.lastSegment();
                    //Compute area
                    var a = this.measureArea(seg2);
                    this.aAreas.push(a);
                    this.cumulativeArea += a;
                    this.lastArea = 0;
                    //draw polygon
                    this.drawFeature(seg2, this.Polygon, 2); 
                }
                if(this.mType == EmType.distance || this.mType == EmType.both){
                    var seg = this.distPolyline.lastSegment();                 
                    seg.normalStyle=this.distanceNormalStyle;
                    //Compute distance
                    var d = this.measureSegment(seg);
                    this.aDistances.push(d);
                    this.cumulativeDistance += d;
                    this.lastDistance = 0;
                    //draw polyline                             
                    this.drawFeature(seg, this.distPolyline, 2);
                } 
                this.triggerEvent(Fusion.Event.RULER_DISTANCE_CHANGED, this, this.getDistance());
                if (this.measureTip) {
                    this.updateTip(e);
                } 
            }
        }
    },

    /**
     * (public) mouseMove(e)
     *
     * handle the mouse move event
     *
     * @param e Event the event that happened on the mapObj
     */
    mouseMove: function(e) {
        if (!this.isDigitizing) {
            return;
        }
        var oElement = this.getMap().getDomObj();
        var target = e.target || e.srcElement;
        if (target.id != '_oEventDiv_'+oElement.id) {
            return;
        }       
        if(this.mType == EmType.area || this.mType == EmType.both){
            var seg2 = this.Polygon.lastSegment();
            seg2.editStyle=this.polygonEditStyle;
            //update area
            this.updateArea(seg2, e);
            //draw polygon
            this.drawFeature(seg2, this.Polygon, 3, e);      
        }
        if(this.mType == EmType.distance || this.mType == EmType.both){  
            //update distance
            var seg = this.distPolyline.lastSegment();
            seg.editStyle=this.distanceEditStyle;
            this.updateDistance(seg, e);      
            //draw line
            this.drawFeature(seg, this.distPolyline, 3, e);
        }
    },
   
    /**
     * (public) dblClick(e)
     *
     * handle the mouse dblclick event
     *
     * @param e Event the event that happened on the mapObj
     */
    dblClick: function(e) {
        //console.log('Digitizer.dblClick');
        if (!this.isDigitizing) return;
        //draw polygon
        if(this.mType == EmType.area || this.mType == EmType.both){
          var seg2 = this.Polygon.lastSegment();
          this.drawFeature(seg2, this.Polygon, 4, e);     
        }
        //draw line
        if(this.mType == EmType.distance || this.mType == EmType.both){
          var seg = this.distPolyline.lastSegment();
          this.drawFeature(seg, this.distPolyline, 4, e);
        }  
        this.isDigitizing = false;
    },
    
    /**
     * (public) measureSegment(seg)
     *
     * return the length of the given segment
     *
     * @param seg Current canvas segment object
     * @return length of the given segment
     */    
    measureSegment: function(seg) {
        var map = this.getMap();
        var dist = Math.sqrt(Math.pow(seg.to.x-seg.from.x,2) +
                         Math.pow(seg.to.y-seg.from.y,2));        
        var dist = this.toFieldUnit(dist);
        return dist;
    },

    /**
     * (public) toFieldUnit(dist)
     *
     * return a distance in field unit, used both for area and distance computing
     *
     * @param dist Current distance
     * @return distance in field unit
     */      
    toFieldUnit: function(dist) {
    	var map = this.getMap();
        var d = dist;
    	if (this.units != Fusion.PIXELS) {
            d = dist * map._fMetersperunit;
        }
        /* magic number - this means the map units are meters! */
        if (map._fMetersperunit == 111319.4908) {
            var center = map.getCurrentCenter();
            d = d * Math.cos(2 * Math.PI * center.y / 360);
        }
    	return d;
    },

    /**
     * (public) measureArea(seg)
     *
     * Compute partial area between the given segment and the first area point,
     * positive when the segment (vector) turns clockwise around the first point.
     * The total area is the sum of all partial areas.
     * Segment area must be signed for non-convex area computing.
     * Intersecting segment result on false area
     * @param seg Current canvas segment object
     * @return signed area
     */    
    measureArea: function(seg) {
        var map = this.getMap();
        var a = 0;
        var tmpArray = [];
	//Initialise an array with the first point of area and the given segment, formula ends with first point
	tmpArray[0] = this.aAreaFirstPoint;
	tmpArray[1] = seg.from;
	tmpArray[2] = seg.to;
	tmpArray[3] = this.aAreaFirstPoint;
	    for ( var i = 0; i < 3; i++ ){
	        a += this.toFieldUnit(tmpArray[i+1].x - tmpArray[i].x)*this.toFieldUnit(tmpArray[i].y + tmpArray[i+1].y - 2*tmpArray[0].y)/2;
	    }   
        //return signed area
        return a;             
    },
    
     /**
     * (public) updateDistance(seg, e)
     *
     * Update distance tip and value when onMouseOver Event occurs
     *
     * @param seg Current segment
     * @param e Event The event that happened on the mapObj
     */ 
    updateDistance: function(seg, e) {
        this.lastDistance = this.measureSegment(seg);
        this.triggerEvent(Fusion.Event.RULER_DISTANCE_CHANGED, this, this.getDistance());
        if (this.measureTip) {
            this.updateTip(e);
        }
    },
    
    getLastDistance: function() {
        var d = this.lastDistance;
        if (this.units != Fusion.PIXELS && this.units != Fusion.METERS) {
            d = Fusion.fromMeter(this.units,d);
        }
        return d;
    },
    
    getDistance: function() {
        var totalDistance = this.cumulativeDistance + this.lastDistance;
        if (this.units != Fusion.PIXELS && this.units != Fusion.METERS) {
            totalDistance = Fusion.fromMeter(this.units,totalDistance);
        }
        return totalDistance;
    },

     /**
     * (public) updateArea(seg, e)
     *
     * Update area tip and value when onMouseOver Event occurs
     *
     * @param seg Current segment
     * @param e Event The event that happened on the mapObj
     */  
    updateArea: function(seg, e){
        this.lastArea = this.measureArea(seg);
        //AS this event changes distances and area too, keep this name for now...
        this.triggerEvent(Fusion.Event.RULER_DISTANCE_CHANGED, this, this.getArea());
        if (this.measureTip) {
            this.updateTip(e);
        }
    },
   
    getLastArea: function() {
        return this.lastArea;
    },

    getArea: function() {
        return this.cumulativeArea + this.lastArea;
    },

    setParameter: function(param, value) {
      //console.log('setParameter: ' + param + ' = ' + value);
        if (param == 'Units') {
            this.units = Fusion.unitFromName(value);
        }
    },

     /**
     * (public) updateTip()
     *
     * Update distance and/or area tip and value
     *
     */     
    updateTip: function(e) {
        if (!this.measureTip) {
            return;
        }
        //for dist
        if(this.mType == EmType.distance || this.mType == EmType.both){
            var segDistance = this.getLastDistance();
            var totalDistance = this.getDistance();
            var dAmp = Math.pow(10, this.distPrecision);
            segDistance = parseInt(segDistance * dAmp)/dAmp;
            totalDistance = parseInt(totalDistance * dAmp)/dAmp;
        }
        if(this.mType == EmType.area || this.mType == EmType.both){
            var segArea = this.getLastArea();
            var totalArea = this.getArea();
            var aAmp = Math.pow(10, this.areaPrecision);
            segArea = parseInt(segArea * aAmp)/aAmp;
            totalArea = parseInt(totalArea * aAmp)/aAmp;
        }  
        if ((segDistance == 0 && totalDistance == 0)||((segArea == 0 && totalArea == 0) && this.mType != EmType.both))
        {
                this.measureTip.innerHTML = '';
                if(this.tooltipType == 'dynamic' ||this.tooltipType == 'static' ) 
                {
                    this.measureTip.style.display = 'none';
                }     
        } 
        else 
        {  
            if(Fusion.unitName(this.units) == "Meters" || !this.units)
            var unitDisplay = " m";
            //var dTipStr = "<table><tr><td align='left' style='font-weight: bold'>Segment: </td><td>&nbsp</td><td>" + segDistance + unitDisplay + "</td>" + "<tr><td align='left' style='font-weight: bold'>Total: </td><td>&nbsp</td><td>" + totalDistance + unitDisplay +"</td></tr></table>";
            var dTipStr ="<TABLE><TR><TH>Segm:</TH><TD>" + segDistance + unitDisplay + "</TD></TR><TR><TH>Total:</TH><TD>" + totalDistance + unitDisplay + "</TD></TR></TABLE>";
            var aTipStr = "<TABLE><TR><TH>Area:</TH><TD>" + segArea + unitDisplay + " </TD></TR><TR><TH>Total:</TH><TD>" + totalArea + unitDisplay + " </TD></TR></TABLE>";
            
            // Distance or both
            if(this.mType == EmType.distance){
                this.measureTip.innerHTML = dTipStr;               
            }
            // Area or both
            if(this.mType == EmType.area){
                this.measureTip.innerHTML = aTipStr;
            }
            if(this.mType == EmType.both){
                this.measureTip.innerHTML = dTipStr + aTipStr;
            }
            // Both, area and distance
            if(this.tooltipType == 'dynamic') 
            {
                this.measureTip.style.display = 'block';
                this.measureTip.style.visibility = 'visible';
                var p = this.getMap().getEventPosition(e);
                var size = Element.getDimensions(this.measureTip);
                var t = (p.y - size.height * 1.5);
                if (t < 0) {
                    t = p.y + size.height * 0.5;
                }               
                var pSize = Element.getDimensions(this.measureTip.parentNode);
                var l = p.x;
                if (l+size.width > pSize.width) {
                    l = p.x - size.width;
                }       
                this.measureTip.style.top = t + 'px';
                this.measureTip.style.left = l + 'px';
            }//end dynamic
            if(this.tooltipType == 'static') 
            {
                this.measureTip.style.display = 'block';
                this.measureTip.style.visibility = 'visible';
                this.measureTip.style.top = this.measureTipPositionTop + 'px';
                this.measureTip.style.left = this.measureTipPositionLeft + 'px';
            }//end static
          }//else
    },
    
};
